****************************************
****Chapter 10: Higher-level models*****
****************************************


*** 10.2 Do expiratory flow measurements vary between methods?

use http://www.stata-press.com/data/mlmus2/pefr, clear

reshape long wm wp, i(id) j(occasion) 
generate i = _n 
reshape long w, i(i) j(meth) string 
sort id meth occasion
list id meth occasion w in 1/8, clean noobs

encode meth, gen(method) 
recode method 2=0


*** 10.3 Two-level variance components models

xtmixed w || id:, mle
estimates store model1

xtmixed w method || id:, mle
estimates store model2

twoway (scatter w id if method==0, msymbol(circle))                 ///
       (scatter w id if method==1, msymbol(circle_hollow)),         ///
       xtitle(Subject id) ytitle(Peak-expiratory-flow measurements) ///
       legend( order(1 "Wright" 2 "Mini Wright")) xlabel(1/17) 

*** 10.4 Three-level variance-components models

** 10.4.4 Estimation using xtmixed

xtmixed w method || id: || method:, mle
estimates store model3

lrtest model2 model3

xtmixed w || id: || method:, mle

** 10.4.5 Empirical Bayes prediction using xtmixed

predict subj instr, reffects
sort id method
label define m 0 "Wright" 1 "Mini Wright"
label values method m
list id method subj instr if id<8 & occasion==1, clean noobs



*** 10.7 Estimation of three-level random-intercepts model using Stata

use http://www.stata-press.com/data/mlmus2/guatemala, clear

** 10.7.1 Using gllamm 
gllamm immun kid2p indNoSpa indSpa  momEdPri momEdSec husEdPri husEdSec  ///
   husEdDK rural pcInd81, family(binomial) link(logit) i(mom cluster)    ///
   nip(5)

matrix a = e(b)
gllamm immun kid2p indNoSpa indSpa  momEdPri momEdSec husEdPri husEdSec  ///
   husEdDK rural pcInd81, family(binomial) link(logit) i(mom cluster)    ///
   from(a) adapt
   
gllamm, eform

estimates store glri8

display exp(sqrt(2*(1.03+5.19))*invnorm(3/4))
display exp(sqrt(2*(5.19))*invnorm(3/4))


** 10.7.2 Using xtmelogit 
* Laplace 
xtmelogit immun kid2p indNoSpa indSpa momEdPri momEdSec husEdPri husEdSec ///
   husEdDK rural pcInd81 || cluster: || mom:, intpoints(1)

* Adaptive quadrature
matrix a = e(b)
xtmelogit immun kid2p indNoSpa indSpa momEdPri momEdSec husEdPri husEdSec ///
   husEdDK rural pcInd81 || cluster: || mom:, intpoints(5)                ///
   from(a) refineopts(iterate(0))

matrix a = e(b)
xtmelogit immun kid2p indNoSpa indSpa momEdPri momEdSec husEdPri husEdSec ///
   husEdDK rural pcInd81 || cluster: || mom:, intpoints(8)                ///
   from(a) refineopts(iterate(0))


*** 10.9 Estimation of three-level random-coefficients model using Stata

** 10.9.1 Using gllamm 
* Three-level random intercept model

estimates restore glri8
matrix a = e(b)
gllamm immun kid2p rural pcInd81, family(binomial) link(logit) ///
   i(mom cluster) from(a) skip adapt eform

estimates store glri0

* Three-level random coefficient model
generate cons = 1
eq inter: cons
eq slope: kid2p
matrix a = e(b)
matrix a = (a,.2,0)
gllamm immun kid2p rural pcInd81, family(binomial) link(logit) i(mom cluster) ///
   nrf(1 2) eqs(inter inter slope) nip(8 4 4) from(a) copy adapt eform
estimates store glrc

lrtest glrc glri0

** 10.9.2 Using xtmelogit 

xtmelogit immun kid2p rural pcInd81 || cluster: kid2p, cov(unstructured) || mom: ///
   intpoints(4 8) or
estimates store xtrc

*** 10.10 Prediction of random effects

** 10.10.1 Empirical Bayes prediction

estimates restore glrc
gllapred zeta, u
sort cluster mom
list cluster mom zetam1 zetam2 zetam3 in 1/9, clean noobs

egen pick_com = tag(cluster)
twoway scatter zetam3 zetam2 if pick_com==1, xtitle(Intercept) ytitle(Slope)

** 10.10.2 Empirical Bayes modal prediction

estimates restore xtrc
predict comms commi mother, reffects

sort cluster mom
list cluster mom mother commi comms in 1/9, clean noobs


*** 10.11 Different kinds of predicted probabilities

** 10.11.1 Predicted marginal probabilities
estimates restore glrc
gllapred margp, mu marginal

** 10.11.2 Predicted median or conditional probabilities
generate z1 = 0
generate z2 = 0
generate z3 = 0
gllapred condp, mu us(z)

label define r 0 "Urban" 1 "Rural"
label values rural r
twoway (line condp pcInd81 if kid2p==0, lpatt(solid) sort) ///
       (line condp pcInd81 if kid2p==1, lpatt(solid) sort) ///
       (line margp pcInd81 if kid2p==0, lpatt(dash) sort) ///
       (line margp pcInd81 if kid2p==1, lpatt(dash) sort), ///
        by(rural) legend(order(1 "Conditional" 3 "Marginal")) ///
        xtitle(Perentage Indigenous) ytitle(Probability)

** 10.11.3 Predicted posterior mean probabilities
gllapred postp, mu
sort cluster mom
list cluster mom kid2p zetam1 zetam2 zetam3 postp in 1/9, clean noobs

