***********************************************
****Chapter 8: Discrete-time survival data*****
***********************************************

*** 8.2 Single-level models for discrete-time survival data

** 8.2.1 Discrete-time hazard and discrete-time survival

use http://www.stata-press.com/data/mlmus2/promotion, clear

ltable dur event, noadjust
ltable dur event, hazard noadjust

** 8.2.2 Data expansion for disrete-time survival analysis
 
list id dur event if id<3

expand dur
by id, sort: generate year = _n
list id dur year event if id<3, sepby(id)
generate y = 0
replace y = event if year==dur

** 8.2.3 Estimation via regression models for dichotomous responses

tabulate year y, row

xi: logit y i.year
predict haz, pr

list id year haz y event if id<3, sepby(id)

twoway (line haz year if id==1, connect(stairstep)), ///
       legend(off) xtitle(Year) ytitle(Discrete-time hazard)

** 8.2.4 Including covariates
* Time-constant covariates

xi: logit y i.year undgrad phdmed phdprest

predict lo, xb

twoway (line lo year if id==1, connect(stairstep) lpatt(solid)) ///
       (line lo year if id==4, connect(stairstep) lpatt(dash)), ///
       legend(off) xtitle(Year) ytitle(Log odds)

generate ln_one_m_haz = ln(1-invlogit(lo))
by id (year), sort: generate ln_surv = sum(ln_one_m_haz)
generate surv = exp(ln_surv)

twoway (line surv year if id==1, connect(stairstep) lpatt(solid)) ///
       (line surv year if id==4, connect(stairstep) lpatt(dash)), ///
       legend(off) xtitle(Year) ytitle(Survival)

* Time-varying covariates

use http://www.stata-press.com/data/mlmus2/promotion, clear

reshape long art cit, i(id) j(year)
list id year art cit dur event if id<3, sepby(id)

drop if year>dur
generate y = 0
replace y = event if year==dur
generate prestige = prest1
replace prestige = prest2 if year>=jobtime

xi: logit y i.year undgrad phdmed phdprest art cit prestige, or




*** 8.4 Data expansion

use http://www.stata-press.com/data/mlmus2/mortality, clear

egen discrete = cut(time), at(0 1 6 12 24 61) icodes
table discrete, contents(min time max time)

replace discrete = discrete + 1

ltable discrete death, noadjust 

expand discrete
by kidid, sort: generate interval = _n
generate y = 0
replace y = death if interval == discrete
tabulate interval y, row 


*** 8.6 Complementary log-log models

quietly tabulate interval, generate(int)
cloglog y int1-int5, nocons 

predict haz, pr

sort kidid interval
list kidid interval haz if kidid==101
 
generate mage2 = mage^2

generate comp12 = f0011*(interval==4)
generate comp24e = f0011*(interval==5)
generate comp24l = f1223*(interval==5)

cloglog y int1-int5 mage mage2 border p0014 p1523 p2435 p36up ///
   pdead comp12 comp24e comp24l, noconstant eform vce(cluster momid) 


*** 8.7 A random-intercept complementary log-log model

** 8.7.2 Estimation using Stata

xtcloglog y int1-int5 mage mage2 border p0014 p1523 p2435 p36up ///
    pdead comp12 comp24e comp24l, noconstant eform i(momid)

*** 8.8 Marginal and conditional survival probabilities

gllamm y int1-int5 mage mage2 border p0014 p1523 p2435 p36up         ///
   pdead comp12 comp24e comp24l, noconstant eform i(momid) link(cll) ///
   family(binom) adapt

keep if kidid==101
replace pdead = 0
replace p2435 = 0
save junk, replace

replace kidid = 1
append using junk
replace kidid = 2 if kidid==101
append using junk
replace kidid = 3 if kidid==101
append using junk
replace kidid = 4 if kidid==101
append using junk
replace kidid = 5 if kidid==101

drop if interval>kidid

sort kidid interval
list kidid interval y, sepby(kidid)

replace momid = kidid

gllapred loglik, ll fsample
generate msurv = exp(loglik)

generate zeta1 = 0
gllapred chaz, mu us(zeta) fsample

generate ln1mchaz = ln(1-chaz)
by kidid (interval), sort: generate sln1mchaz = sum(ln1mchaz)
generate csurv = exp(sln1mchaz)

twoway (line msurv interval if kidid==interval, sort lpatt(solid))  ///
       (line csurv interval if kidid==5, sort lpatt(dash)),         ///
       legend(order(1 "marginal" 2 "conditional"))                  ///
       ytitle(Survival probability)

