**************************
****Chapter 9: Counts*****
**************************

*** 9.5 Longitudinal data structure

use http://www.stata-press.com/data/mlmus2/drvisit, clear

xtdescribe if numvisit<., i(id) t(reform)

*** 9.6 Single-level Poisson regression
poisson numvisit reform age educ married badh loginc summer, irr
estimates store ordinary 

glm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) eform


*** 9.7 Random-intercept Poisson regression
** 9.7.2 Estimation using Stata 
* Using xtpoisson
xtpoisson numvisit reform age educ married badh loginc summer, ///
 i(id) normal irr 

* Using xtmepoisson
xtmepoisson numvisit reform age educ married badh loginc summer ///
  || id:, irr
estimates store xtri

* Using gllamm
generate cons=1
eq ri: cons
gllamm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) i(id) eqs(ri) eform adapt 
estimates store glri

display sqrt(.81691979)

gllamm, robust eform

lrtest ordinary glri, force


*** 9.8 Random-coefficient Poisson regression 

** 9.8.2 Estimation using Stata
* Using xtmepoisson
xtmepoisson numvisit reform age educ married badh loginc summer ///
  || id: reform, cov(unstructured) irr
estimates store xtrc
lrtest xtri xtrc

* Using gllamm
estimates restore glri
matrix a = e(b)
eq rc: reform
gllamm numvisit reform age educ married badh loginc summer, ///
  family(poisson) link(log) i(id) nrf(2) eqs(ri rc) from(a) eform adapt 
estimates store glrc

gllamm, robust eform


*** 9.9 Overdispersion in single-level models

** 9.9.1 Normally distributed random intercept 
generate obs = _n
xtpoisson numvisit reform age educ married badh loginc summer, ///
   i(obs) normal irr 

*** 9.9.2 Negative binomial models
* Mean dispersion or NB2
nbreg numvisit reform age educ married badh loginc summer, irr ///
   dispersion(mean)
* Constant dispersion or NB1
nbreg numvisit reform age educ married badh loginc summer, irr ///
   dispersion(constant)

*** 9.9.3 Quasilikelihood or robust standard errors 
glm numvisit reform age educ married badh loginc summer, ///
 family(poisson) link(log) eform scale(x2)


*** 9.10 Level-1 overdispersion in two-level models

estimates restore glri

gllamm, robust eform


*** 9.11 Other approaches to two-level count data

** 9.11.3 Generalized estimating equations (GEE)
xtgee numvisit reform age educ married badh loginc summer, ///
  i(id) family(poisson) link(log) robust eform
 
** 9.11.4 Comparing marginal and conditional estimates when responses are missing at random
egen num = count(numvisit), by(id)
generate mult = 3 - num
expand mult
by id (reform), sort: generate occ = _n - 1
generate missing = (occ!=reform & num==1)
replace numvisit = . if missing==1
replace reform = occ if missing==1

sort id reform
set seed 1211

estimates restore glri
gllasim y, fsample

by id (reform), sort: generate drop = (y[1]>2 & uniform()<.9)
replace y = . if drop==1 & reform==1

xtpoisson y reform age educ married badh loginc summer, ///
 i(id) normal irr 
 
poisson y reform age educ married badh loginc summer, irr

xtgee y reform age educ married badh loginc summer, ///
  i(id) family(poisson) link(log) vce(robust) eform

*** 9.12 Piecewise exponential survival model

use http://www.stata-press.com/data/mlmus2/mortality, clear

list kidid time death if momid==26, noobs

** 9.12.1 No covariates or random effects

stset time, failure(death) id(kidid)

stsplit dur, at(0 1 6 12 24 60)

generate length = _t - _t0
table dur, contents(sum length sum _d)

quietly tabulate dur, generate(ageband)

poisson _d ageband1-ageband5, exposure(length) noconstant irr


** 9.12.2 Including covariates and a random intercept

generate mage2 = mage^2
generate comp12 = f0011*(ageband4==1)
generate comp24e = f0011*(ageband5==1)
generate comp24l = f1223*(ageband5==1)

poisson _d ageband1-ageband5 mage mage2 border pdead p0014 p1523 ///
   p2435 p36up comp12 comp24e comp24l, exposure(length) nocons irr

xtpoisson _d ageband1-ageband5 mage mage2 border pdead p0014 p1523 ///
   p2435 p36up comp12 comp24e comp24l, i(momid) exposure(length) nocons irr




*** 9.14 Standardized mortality ratios

use http://www.stata-press.com/data/mlmus2/lips, clear
generate smr = 100*o/e


*** 9.15 Random-intercept Poisson regression

** 9.15.2 Estimation using Stata
generate lne = ln(e)
gllamm o, i(county) offset(lne) f(poiss) adapt


** 9.15.3 Prediction of SMRs
gllapred mu, mu nooffset
generate thet = 100*mu
sort county
list county thet in 1/10, clean noobs

twoway (scatter thet smr, msymbol(none) mlabpos(0) mlabel(county)) ///
 (function y=x, range(0 600)), xline(108) yline(108)               ///
 xtitle(Crude SMR) ytitle(Empirical Bayes SMR) legend(off)
 

*** 9.16 Nonparametric maximum likelihood estimation

** 9.16.2 Estimation using gllamm
gllamm o, i(county) offset(lne) family(poisson) ip(f) nip(2) 

matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) ip(f) nip(3) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')

matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) ip(f) nip(4) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')
  
matrix a = e(b)
local ll = e(ll)
local k = e(k)

gllamm o, i(county) offset(lne) family(poisson) ip(f) nip(5) ///
  gateaux(-5 5 100) from(a) lf0(`k' `ll')
  
matrix locs = e(zlc2)'
matrix lp = e(zps2)'
svmat locs
svmat lp

generate p = exp(lp1)
generate smrloc = 100*exp(_b[_cons] + locs1)
twoway (dropline p smrloc), xtitle(Location) ytitle(Probability)

** 9.16.3 Prediction
gllapred mu2, mu nooffset
generate thet2 = 100*mu2
sort county
list county thet thet2 in 1/10, clean noobs

